Documentation
¶
Overview ¶
Package cqlx makes working with Cassandra and Scylla easy and error prone without sacrificing performance. It’s inspired by Sqlx, a tool for working with SQL databases, but it goes beyond what Sqlx provides.
cqlx is built on top of github.com/apache/cassandra-gocql-driver/v2 — the upstream Apache driver — and is a fork of github.com/scylladb/gocqlx with an identical public API.
For more details consult README.
Index ¶
- Variables
- func CompileNamedQuery(qs []byte) (stmt string, names []string, err error)
- func CompileNamedQueryString(qs string) (stmt string, names []string, err error)
- type Batch
- func (b *Batch) Bind(qry *Queryx, args ...any) error
- func (b *Batch) BindMap(qry *Queryx, arg map[string]any) error
- func (b *Batch) BindStruct(qry *Queryx, arg any) error
- func (b *Batch) BindStructMap(qry *Queryx, arg0 any, arg1 map[string]any) error
- func (b *Batch) Consistency(c gocqlv2.Consistency) *Batch
- func (b *Batch) DefaultTimestamp(enable bool) *Batch
- func (b *Batch) Observer(observer gocqlv2.BatchObserver) *Batch
- func (b *Batch) Query(stmt string, args ...any) *Batch
- func (b *Batch) RetryPolicy(policy gocqlv2.RetryPolicy) *Batch
- func (b *Batch) SerialConsistency(cons gocqlv2.Consistency) *Batch
- func (b *Batch) SetKeyspace(keyspace string) *Batch
- func (b *Batch) SpeculativeExecutionPolicy(policy gocqlv2.SpeculativeExecutionPolicy) *Batch
- func (b *Batch) Trace(trace gocqlv2.Tracer) *Batch
- func (b *Batch) WithContext(ctx context.Context) *Batch
- func (b *Batch) WithNowInSeconds(now int) *Batch
- func (b *Batch) WithTimestamp(timestamp int64) *Batch
- type Iterx
- type Queryx
- func (q *Queryx) Bind(v ...any) *Queryx
- func (q *Queryx) BindMap(arg map[string]any) *Queryx
- func (q *Queryx) BindStruct(arg any) *Queryx
- func (q *Queryx) BindStructMap(arg0 any, arg1 map[string]any) *Queryx
- func (q *Queryx) Consistency(c gocqlv2.Consistency) *Queryx
- func (q *Queryx) CustomPayload(customPayload map[string][]byte) *Queryx
- func (q *Queryx) DefaultTimestamp(enable bool) *Queryx
- func (q *Queryx) Err() error
- func (q *Queryx) Exec() error
- func (q *Queryx) ExecCAS() (applied bool, err error)
- func (q *Queryx) ExecCASRelease() (bool, error)
- func (q *Queryx) ExecRelease() error
- func (q *Queryx) Get(dest any) error
- func (q *Queryx) GetCAS(dest any) (applied bool, err error)
- func (q *Queryx) GetCASRelease(dest any) (bool, error)
- func (q *Queryx) GetRelease(dest any) error
- func (q *Queryx) Idempotent(value bool) *Queryx
- func (q *Queryx) Iter() *Iterx
- func (q *Queryx) NoSkipMetadata() *Queryx
- func (q *Queryx) Observer(observer gocqlv2.QueryObserver) *Queryx
- func (q *Queryx) PageSize(n int) *Queryx
- func (q *Queryx) PageState(state []byte) *Queryx
- func (q *Queryx) Prefetch(p float64) *Queryx
- func (q *Queryx) RetryPolicy(r gocqlv2.RetryPolicy) *Queryx
- func (q *Queryx) RoutingKey(routingKey []byte) *Queryx
- func (q *Queryx) Scan(v ...any) error
- func (q *Queryx) Select(dest any) error
- func (q *Queryx) SelectRelease(dest any) error
- func (q *Queryx) SerialConsistency(cons gocqlv2.SerialConsistency) *Queryx
- func (q *Queryx) SetHostID(hostID string) *Queryx
- func (q *Queryx) SetKeyspace(keyspace string) *Queryx
- func (q *Queryx) SetSpeculativeExecutionPolicy(sp gocqlv2.SpeculativeExecutionPolicy) *Queryx
- func (q *Queryx) Strict() *Queryx
- func (q *Queryx) Trace(trace gocqlv2.Tracer) *Queryx
- func (q *Queryx) WithBindTransformer(tr Transformer) *Queryx
- func (q *Queryx) WithContext(ctx context.Context) *Queryx
- func (q *Queryx) WithNowInSeconds(now int) *Queryx
- func (q *Queryx) WithTimestamp(timestamp int64) *Queryx
- type Session
- func (s *Session) Batch(bt gocqlv2.BatchType) *Batch
- func (s *Session) ContextBatch(ctx context.Context, bt gocqlv2.BatchType) *Batch
- func (s Session) ContextQuery(ctx context.Context, stmt string, names []string) *Queryx
- func (s Session) ExecStmt(stmt string) error
- func (s *Session) ExecuteBatch(batch *Batch) error
- func (s *Session) ExecuteBatchCAS(batch *Batch, dest ...any) (applied bool, iter *gocqlv2.Iter, err error)
- func (s *Session) MapExecuteBatchCAS(batch *Batch, dest map[string]any) (applied bool, iter *gocqlv2.Iter, err error)
- func (s *Session) NewBatch(bt gocqlv2.BatchType) *Batchdeprecated
- func (s Session) Query(stmt string, names []string) *Queryx
- type Transformer
- type UDT
Examples ¶
Constants ¶
This section is empty.
Variables ¶
var DefaultMapper = reflectx.NewMapperFunc("db", reflectx.CamelToSnakeASCII)
DefaultMapper uses `db` tag and automatically converts struct field names to snake case. It can be set to whatever you want, but it is encouraged to be set before cqlx is used as name-to-field mappings are cached after first use on a type.
A custom mapper can always be set per Sessionm, Query and Iter.
var DefaultStrict bool
DefaultStrict disables the behavior of forcing queries and iterators to ignore missing fields for all queries. See Strict below for more information.
var UnsetEmptyTransformer = func(name string, val any) any { v := reflect.ValueOf(val) if v.IsZero() { return gocqlv2.UnsetValue } return val }
UnsetEmptyTransformer unsets all empty parameters. It helps to avoid tombstones when using the same insert/update statement for filled and partially filled named parameters.
Functions ¶
func CompileNamedQuery ¶
CompileNamedQuery translates query with named parameters in a form ':<identifier>' to query with '?' placeholders and a list of parameter names. If you need to use ':' in a query, i.e. with maps or UDTs use '::' instead.
func CompileNamedQueryString ¶
CompileNamedQueryString translates query with named parameters in a form ':<identifier>' to query with '?' placeholders and a list of parameter names. If you need to use ':' in a query, i.e. with maps or UDTs use '::' instead.
Types ¶
type Batch ¶
Batch is a wrapper around gocqlv2.Batch
func (*Batch) Bind ¶
Bind binds query parameters to values from args. If value cannot be found an error is reported.
func (*Batch) BindMap ¶
BindMap binds query named parameters to values from arg using a mapper. If value cannot be found an error is reported.
func (*Batch) BindStruct ¶
BindStruct binds query named parameters to values from arg using a mapper. If value cannot be found an error is reported.
func (*Batch) BindStructMap ¶
BindStructMap binds query named parameters to values from arg0 and arg1 using a mapper. If value cannot be found an error is reported.
func (*Batch) Consistency ¶
func (b *Batch) Consistency(c gocqlv2.Consistency) *Batch
Consistency sets the consistency level for this batch. If no consistency level has been set, the default consistency level of the cluster is used.
func (*Batch) DefaultTimestamp ¶
DefaultTimestamp will enable the with default timestamp flag on the query. If enabled, this will replace the server side assigned timestamp as default timestamp. Note that a timestamp in the query itself will still override this timestamp. This is entirely optional.
Only available on protocol >= 3
func (*Batch) Observer ¶
func (b *Batch) Observer(observer gocqlv2.BatchObserver) *Batch
Observer enables batch-level observer on this batch. The provided observer will be called every time this batched query is executed.
func (*Batch) RetryPolicy ¶
func (b *Batch) RetryPolicy(policy gocqlv2.RetryPolicy) *Batch
RetryPolicy sets the retry policy to use when executing the batch operation
func (*Batch) SerialConsistency ¶
func (b *Batch) SerialConsistency(cons gocqlv2.Consistency) *Batch
SerialConsistency sets the consistency level for the serial phase of conditional updates. That consistency can only be either SERIAL or LOCAL_SERIAL and if not present, it defaults to SERIAL. This option will be ignored for anything else that a conditional update/insert.
Only available for protocol 3 and above
func (*Batch) SetKeyspace ¶
SetKeyspace will enable per-batch keyspace override (Cassandra 5.0+).
func (*Batch) SpeculativeExecutionPolicy ¶
func (b *Batch) SpeculativeExecutionPolicy(policy gocqlv2.SpeculativeExecutionPolicy) *Batch
SpeculativeExecutionPolicy sets the speculative execution policy to use when executing the batch operation
func (*Batch) Trace ¶
Trace enables tracing of this batch. Look at the documentation of the gocqlv2.Tracer interface to learn more about tracing.
func (*Batch) WithContext ¶
WithContext returns a copy of b with its context set to ctx. The context controls the entire lifetime of executing the batch: it will be canceled and return once the context is canceled.
Note: unlike gocqlx, the returned *Batch shares the underlying *gocql.Batch with the original — Apache v2 deprecated Batch.WithContext, and cqlx stores the context on the wrapper to keep the public API gocqlx-compatible. As a result, subsequent mutations (e.g. Query, RetryPolicy) on either copy are observed by both. See docs/api-deltas.md.
func (*Batch) WithNowInSeconds ¶
WithNowInSeconds sets the "now" value (in seconds) used by the server to evaluate TTLs and write times for this batch. Available on protocol >= 5.
func (*Batch) WithTimestamp ¶
WithTimestamp will enable the with default timestamp flag on the query like DefaultTimestamp does. But also allows to define value for timestamp. It works the same way as USING TIMESTAMP in the query itself, but should not break prepared query optimization.
Only available on protocol >= 3
type Iterx ¶
type Iterx struct {
*gocqlv2.Iter
Mapper *reflectx.Mapper
// contains filtered or unexported fields
}
Iterx is a wrapper around gocqlv2.Iter which adds struct scanning capabilities.
func (*Iterx) Close ¶
Close closes the iterator and returns any errors that happened during the query or the iteration.
func (*Iterx) Get ¶
Get scans first row into a destination and closes the iterator.
If the destination type is a struct pointer, then StructScan will be used. If the destination is some other type, then the row must only have one column which can scan into that type. This includes types that implement gocqlv2.Unmarshaler and gocqlv2.UDTUnmarshaler.
If you'd like to treat a type that implements gocqlv2.Unmarshaler or gocqlv2.UDTUnmarshaler as an ordinary struct you should call StructOnly().Get(dest) instead.
If no rows were selected, ErrNotFound is returned.
func (*Iterx) Scan ¶
Scan consumes the next row of the iterator and copies the columns of the current row into the values pointed at by dest. Use nil as a dest value to skip the corresponding column. Scan might send additional queries to the database to retrieve the next set of rows if paging was enabled.
Scan returns true if the row was successfully unmarshaled or false if the end of the result set was reached or if an error occurred. Close should be called afterwards to retrieve any potential errors.
func (*Iterx) Select ¶
Select scans all rows into a destination, which must be a pointer to slice of any type, and closes the iterator.
If the destination slice type is a struct, then StructScan will be used on each row. If the destination is some other type, then each row must only have one column which can scan into that type. This includes types that implement gocqlv2.Unmarshaler and gocqlv2.UDTUnmarshaler.
If you'd like to treat a type that implements gocqlv2.Unmarshaler or gocqlv2.UDTUnmarshaler as an ordinary struct you should call StructOnly().Select(dest) instead.
If no rows were selected, ErrNotFound is NOT returned.
func (*Iterx) Strict ¶
Strict forces the iterator to disable ignoring missing fields. In Strict mode when scanning a struct if result row has a column that cannot be mapped to any destination field an error is reported. By default such columns are ignored.
func (*Iterx) StructOnly ¶
StructOnly forces the iterator to treat a single-argument struct as non-scannable. This is is useful if you need to scan a row into a struct that also implements gocqlv2.UDTUnmarshaler or in rare cases gocqlv2.Unmarshaler.
func (*Iterx) StructScan ¶
StructScan is like gocqlv2.Iter.Scan, but scans a single row into a single struct. Use this and iterate manually when the memory load of Select() might be prohibitive. StructScan caches the reflect work of matching up column positions to fields to avoid that overhead per scan, which means it is not safe to run StructScan on the same Iterx instance with different struct types.
type Queryx ¶
type Queryx struct {
Mapper *reflectx.Mapper
*gocqlv2.Query
Names []string
// contains filtered or unexported fields
}
Queryx is a wrapper around gocqlv2.Query which adds struct binding capabilities.
func (*Queryx) Bind ¶
Bind sets query arguments of query. This can also be used to rebind new query arguments to an existing query instance.
func (*Queryx) BindStruct ¶
BindStruct binds query named parameters to values from arg using mapper. If value cannot be found error is reported.
func (*Queryx) BindStructMap ¶
BindStructMap binds query named parameters to values from arg0 and arg1 using a mapper. If value cannot be found in arg0 it's looked up in arg1 before reporting an error.
func (*Queryx) Consistency ¶
func (q *Queryx) Consistency(c gocqlv2.Consistency) *Queryx
Consistency sets the consistency level for this query. If no consistency level have been set, the default consistency level of the cluster is used.
func (*Queryx) CustomPayload ¶
CustomPayload sets the custom payload level for this query.
func (*Queryx) DefaultTimestamp ¶
DefaultTimestamp will enable the with default timestamp flag on the query. If enable, this will replace the server side assigned timestamp as default timestamp. Note that a timestamp in the query itself will still override this timestamp. This is entirely optional.
Only available on protocol >= 3
func (*Queryx) ExecCAS ¶
ExecCAS executes the Lightweight Transaction query, returns whether query was applied.
When using Cassandra it may be necessary to use NoSkipMetadata in order to obtain an accurate "applied" value. See the documentation of NoSkipMetaData method on this page for more details.
func (*Queryx) ExecCASRelease ¶
ExecCASRelease calls ExecCAS.
On Apache v2 the underlying driver releases query resources automatically, so this is equivalent to ExecCAS. The method is kept for API symmetry with gocqlx.
func (*Queryx) ExecRelease ¶
ExecRelease calls Exec.
On Apache v2 the underlying driver releases query resources automatically, so this is equivalent to Exec. The method is kept for API symmetry with gocqlx.
func (*Queryx) Get ¶
Get scans first row into a destination and closes the iterator.
If the destination type is a struct pointer, then Iter.StructScan will be used. If the destination is some other type, then the row must only have one column which can scan into that type. This includes types that implement gocqlv2.Unmarshaler and gocqlv2.UDTUnmarshaler.
If you'd like to treat a type that implements gocqlv2.Unmarshaler or gocqlv2.UDTUnmarshaler as an ordinary struct you should call Iter().StructOnly().Get(dest) instead.
If no rows were selected, ErrNotFound is returned.
func (*Queryx) GetCAS ¶
GetCAS executes a lightweight transaction. If the transaction fails because the existing values did not match, the previous values will be stored in dest object.
func (*Queryx) GetCASRelease ¶
GetCASRelease calls GetCAS.
On Apache v2 the underlying driver releases query resources automatically, so this is equivalent to GetCAS. The method is kept for API symmetry with gocqlx.
func (*Queryx) GetRelease ¶
GetRelease calls Get.
On Apache v2 the underlying driver releases query resources automatically, so this is equivalent to Get. The method is kept for API symmetry with gocqlx.
func (*Queryx) Idempotent ¶
Idempotent marks the query as being idempotent or not depending on the value.
func (*Queryx) Iter ¶
Iter returns Iterx instance for the query. It should be used when data is too big to be loaded with Select in order to do row by row iteration. See Iterx StructScan function.
func (*Queryx) NoSkipMetadata ¶
NoSkipMetadata will override the internal result metadata cache so that the driver does not send skip_metadata for queries, this means that the result will always contain the metadata to parse the rows and will not reuse the metadata from the prepared staement. This should only be used to work around cassandra bugs, such as when using CAS operations which do not end in Cas.
func (*Queryx) Observer ¶
func (q *Queryx) Observer(observer gocqlv2.QueryObserver) *Queryx
Observer enables query-level observer on this query. The provided observer will be called every time this query is executed.
func (*Queryx) PageSize ¶
PageSize will tell the iterator to fetch the result in pages of size n. This is useful for iterating over large result sets, but setting the page size too low might decrease the performance. This feature is only available in Cassandra 2 and onwards.
func (*Queryx) PageState ¶
PageState sets the paging state for the query to resume paging from a specific point in time. Setting this will disable to query paging for this query, and must be used for all subsequent pages.
func (*Queryx) Prefetch ¶
Prefetch sets the default threshold for pre-fetching new pages. If there are only p*pageSize rows remaining, the next page will be requested automatically.
func (*Queryx) RetryPolicy ¶
func (q *Queryx) RetryPolicy(r gocqlv2.RetryPolicy) *Queryx
RetryPolicy sets the policy to use when retrying the query.
func (*Queryx) RoutingKey ¶
RoutingKey sets the routing key to use when a token aware connection pool is used to optimize the routing of this query.
func (*Queryx) Scan ¶
Scan executes the query, copies the columns of the first selected row into the values pointed at by dest and discards the rest. If no rows were selected, ErrNotFound is returned.
func (*Queryx) Select ¶
Select scans all rows into a destination, which must be a pointer to slice of any type, and closes the iterator.
If the destination slice type is a struct, then Iter.StructScan will be used on each row. If the destination is some other type, then each row must only have one column which can scan into that type. This includes types that implement gocqlv2.Unmarshaler and gocqlv2.UDTUnmarshaler.
If you'd like to treat a type that implements gocqlv2.Unmarshaler or gocqlv2.UDTUnmarshaler as an ordinary struct you should call Iter().StructOnly().Select(dest) instead.
If no rows were selected, ErrNotFound is NOT returned.
func (*Queryx) SelectRelease ¶
SelectRelease calls Select.
On Apache v2 the underlying driver releases query resources automatically, so this is equivalent to Select. The method is kept for API symmetry with gocqlx.
func (*Queryx) SerialConsistency ¶
func (q *Queryx) SerialConsistency(cons gocqlv2.SerialConsistency) *Queryx
SerialConsistency sets the consistency level for the serial phase of conditional updates. That consistency can only be either SERIAL or LOCAL_SERIAL and if not present, it defaults to SERIAL. This option will be ignored for anything else that a conditional update/insert.
func (*Queryx) SetHostID ¶
SetHostID allows to define the host the query should be executed against. If the host was filtered or otherwise unavailable, then the query will error. If an empty string is sent, the default behavior, using the configured HostSelectionPolicy will be used. A hostID can be obtained from HostInfo.HostID() after calling GetHosts().
func (*Queryx) SetKeyspace ¶
SetKeyspace will enable per-query keyspace override (Cassandra 5.0+).
func (*Queryx) SetSpeculativeExecutionPolicy ¶
func (q *Queryx) SetSpeculativeExecutionPolicy(sp gocqlv2.SpeculativeExecutionPolicy) *Queryx
SetSpeculativeExecutionPolicy sets the execution policy.
func (*Queryx) Strict ¶
Strict forces the query and iterators to report an error if there are missing fields. By default when scanning a struct if result row has a column that cannot be mapped to any destination it is ignored. With strict error is reported.
func (*Queryx) Trace ¶
Trace enables tracing of this query. Look at the documentation of the Tracer interface to learn more about tracing.
func (*Queryx) WithBindTransformer ¶
func (q *Queryx) WithBindTransformer(tr Transformer) *Queryx
WithBindTransformer sets the query bind transformer. The transformer is called right before binding a value to a named parameter.
func (*Queryx) WithContext ¶
WithContext stores ctx on q. The context controls the entire lifetime of executing a query: queries will be canceled and return once the context is canceled. Subsequent Exec/Iter/Scan call the underlying ExecContext/ IterContext/ScanContext driver methods with the stored context.
Note: unlike gocqlx, this does not create a shallow copy of the embedded *gocql.Query. Apache v2 deprecated Query.WithContext in favor of passing context at execution time; cqlx stores the context on the wrapper to keep the public API gocqlx-compatible. See docs/api-deltas.md.
func (*Queryx) WithNowInSeconds ¶
WithNowInSeconds sets the "now" value (in seconds) used by the server to evaluate TTLs and write times for this query. Available on protocol >= 5.
func (*Queryx) WithTimestamp ¶
WithTimestamp will enable the with default timestamp flag on the query like DefaultTimestamp does. But also allows to define value for timestamp. It works the same way as USING TIMESTAMP in the query itself, but should not break prepared query optimization
Only available on protocol >= 3
type Session ¶
Session wraps gocqlv2.Session and provides a modified Query function that returns Queryx instance. The original Session instance can be accessed as Session. The default mapper uses `db` tag and automatically converts struct field names to snake case. If needed package reflectx provides constructors for other types of mappers.
Example ¶
package main
import (
gocqlv2 "github.com/apache/cassandra-gocql-driver/v2"
"github.com/moguchev/cqlx"
"github.com/moguchev/cqlx/qb"
)
func main() {
cluster := gocqlv2.NewCluster("host")
session, err := cqlx.WrapSession(cluster.CreateSession())
if err != nil {
// handle error
}
builder := qb.Select("foo")
session.Query(builder.ToCql())
}
Output:
func NewSession ¶
NewSession wraps existing gocqlv2.session.
func WrapSession ¶
WrapSession should be called on CreateSession() gocql function to convert the created session to cqlx.Session.
Example:
session, err := cqlx.WrapSession(cluster.CreateSession())
func (*Session) ContextBatch ¶
ContextBatch creates a new batch operation using defaults defined in the cluster with attached context.
func (Session) ContextQuery ¶
ContextQuery is a helper function that allows to pass context when creating a query, see the "Query" function .
func (*Session) ExecuteBatch ¶
ExecuteBatch executes a batch operation and returns nil if successful otherwise an error describing the failure.
func (*Session) ExecuteBatchCAS ¶
func (s *Session) ExecuteBatchCAS(batch *Batch, dest ...any) (applied bool, iter *gocqlv2.Iter, err error)
ExecuteBatchCAS executes a batch operation and returns true if successful and an iterator (to scan additional rows if more than one conditional statement) was sent. Further scans on the interator must also remember to include the applied boolean as the first argument to *Iter.Scan
func (*Session) MapExecuteBatchCAS ¶
func (s *Session) MapExecuteBatchCAS(batch *Batch, dest map[string]any) (applied bool, iter *gocqlv2.Iter, err error)
MapExecuteBatchCAS executes a batch operation much like ExecuteBatchCAS, however it accepts a map rather than a list of arguments for the initial scan.
func (Session) Query ¶
Query creates a new Queryx using the session mapper. The stmt and names parameters are typically result of a query builder (package qb) ToCql() function or come from table model (package table). The names parameter is a list of query parameters' names and it's used for binding.
type Transformer ¶
Transformer transforms the value of the named parameter to another value.
var DefaultBindTransformer Transformer
DefaultBindTransformer just do nothing.
A custom transformer can always be set per Query.
type UDT ¶
type UDT interface {
// contains filtered or unexported methods
}
UDT is a marker interface that needs to be embedded in a struct if you want to marshal or unmarshal it as a User Defined Type.
Example ¶
package main
import (
"github.com/moguchev/cqlx"
)
func main() {
// Just add cqlx.UDT to a type, no need to implement marshalling functions
type FullName struct {
cqlx.UDT
FirstName string
LastName string
}
_ = FullName{}
}
Output:
Example (Wraper) ¶
package main
import (
"github.com/moguchev/cqlx"
)
func main() {
type FullName struct {
FirstName string
LastName string
}
// Create new UDT wrapper type
type FullNameUDT struct {
cqlx.UDT
*FullName
}
_ = FullNameUDT{}
}
Output:
Source Files
¶
Directories
¶
| Path | Synopsis |
|---|---|
|
cmd
|
|
|
schemagen
command
|
|
|
Package cqlxtest provides test helpers for integration tests.
|
Package cqlxtest provides test helpers for integration tests. |
|
Package dbutil provides various utilities built on top of core cqlx modules.
|
Package dbutil provides various utilities built on top of core cqlx modules. |
|
Package qb provides CQL query builders.
|
Package qb provides CQL query builders. |
|
Package table adds support for super simple CRUD operations based on table model.
|
Package table adds support for super simple CRUD operations based on table model. |